/* Copyright 2005, 2006, 2007 Columbia University */
/*
 * Definitions of structs and defines for communicating between tsd and 
 * the driver.
 *
 * Authors: 
 * Leo Kim - Network Computing Lab - Columbia University
 */
/* set tabstop = 8 */

#ifndef _THINC_SOUND_H_
#define _THINC_SOUND_H_

/* tsd command constants. These are used to index a function pointer array and
 * to manipulate the tsd. */
typedef enum {
	T_SND_CMD_INIT=0,
	T_SND_CMD_PLAYBACK_OPEN,
	T_SND_CMD_PLAYBACK_END,
	T_SND_CMD_PLAY,
	T_SND_CMD_VOLUME_UP,
	T_SND_CMD_VOLUME_DOWN,
	T_SND_CMD_CAPTURE_OPEN,
	T_SND_CMD_CAPTURE_END,
	T_SND_CMD_LAST
} t_snd_cmd_enum;

/* Sum of all T_SND_CMDs. Make sure this is accurate! */
#define T_SND_NUM_CMDS			T_SND_CMD_LAST+1

/* open mode (capture, playback) */
#define T_MODE_PLAYBACK	0
#define T_MODE_CAPTURE	1

/* For endianness. These are in thincProto.h, but we include them here since 
 * we don't want to bother including that header file here. */
#define T_BIG 		0
#define T_LITTLE 	1

/* Constants */
#define T_SND_PORT			30000
/* Shared buffer size. The idea is that part of this buffer is used for raw pcm
 * data, and the other part is used for control commands. tsd should know how 
 * to read from it accordingly. */
#define T_SND_BUFSIZE			(1024 * 96)	/* 96 kb */
#define T_SND_BLOCKSIZE			1024
#define T_SND_CAPTURE_BLOCKSIZE		1024
#define T_SND_MAX_CAPTURE_BLOCKSIZE	4096
#define T_SND_CMD_BUFSIZE		768
#define T_SND_SOUND_BUFSIZE		T_SND_BUFSIZE - T_SND_CMD_BUFSIZE
#define T_SND_SOUND_BUFSTART		0
#define T_SND_SOUND_BUFEND		T_SND_SOUND_BUFSIZE - 1
#define T_SND_CMD_BUFSTART		T_SND_SOUND_BUFSIZE
#define T_SND_CMD_BUFEND		T_SND_BUFSIZE
#define T_SND_DEVICE			"/dev/tsnd"

/* Device-related constants */
#define T_SND_MAJOR			42
#define T_SND_IOCMAGIC		T_SND_MAJOR
#define T_SND_NAME			"tsnd"

/* IOCTLs */
/* Deletes play command from command queue. */
#define T_SND_IOC_DEL_PLAY_CMD			_IO(T_SND_IOCMAGIC, 1)
/* Reset fifo play commands to 0 */
#define T_SND_IOC_RESET_PLAY_CMDS		_IO(T_SND_IOCMAGIC, 2)
/* Generate thinc_snd_cmd_open ctrl command. */
#define T_SND_IOC_OPEN_CTRL_CMD			_IO(T_SND_IOCMAGIC, 3)
/* Return number of play commands left to read. */
#define T_SND_IOC_NUM_PLAY_CMDS			_IO(T_SND_IOCMAGIC, 4)
/* Retrieve the offset at which to start reading from the play command buffer.*/
#define T_SND_IOC_PLAY_CMD_OFFSET		_IO(T_SND_IOCMAGIC, 5)
/* generate open capture stream command*/
#define T_SND_IOC_OPEN_CAPTURE_STREAM		_IO(T_SND_IOCMAGIC, 6)

/* Commands used for communicating between driver and tsd. */
/* T_SND_CMD_INIT (this one goes from tsd to plugin) */
struct thinc_snd_cmd_init {
	int 		shm_id;
	unsigned int 	shm_buf_sz;
};

/* T_SND_CMD_OPEN */
struct thinc_snd_cmd_open {
	unsigned int bits;
	unsigned int channels;
	unsigned int endian;
	unsigned int rate;
};

/* T_SND_CMD_PLAY */
struct thinc_snd_cmd_play {
	unsigned int offset;	/* offset in sound buffer */
	unsigned int length;
	unsigned int timestamp;
};

/* Main struct */
struct thinc_snd_cmd {
	unsigned int id;
	union {
		struct thinc_snd_cmd_open open;
		struct thinc_snd_cmd_play play;
		struct thinc_snd_cmd_init init;
	} type;
};

/* Macros */
/*
 * min()/max() macros that also do
 * strict type-checking.. See the
 * "unnecessary" pointer comparison.
 */
#define min(x,y) ({ \
		typeof(x) _x = (x); \
		typeof(y) _y = (y); \
		(void) (&_x == &_y);        \
		_x < _y ? _x : _y; })

#define max(x,y) ({ \
		typeof(x) _x = (x); \
		typeof(y) _y = (y); \
		(void) (&_x == &_y);        \
		_x > _y ? _x : _y; })

#endif /* _THINC_SOUND_H_ */
