/* Copyright 2004, 2005, 2007 Columbia University */
/*
 * All-round header file for tsd code. 
 *
 * Authors: 
 * Leo Kim - Network Computing Lab - Columbia University
 * Ricardo Baratto - Network Computing Lab - Columbia University
 */
/* set tabstop = 8 */

#ifndef _TSD_H_

#include <sys/poll.h>
/* for gettimeofday */
#include <time.h>
#include <sys/time.h>

#include "thincCommon.h"
#include "thincProto.h"
#include "thincProtoSound.h"
#include "thinc_ssl.h"
#include "thincSound.h"

/* For version checking on initial handshake */
#define TSD_VERSION_MAJOR	0
#define TSD_VERSION_MINOR	1
#define TSD_PATCHLEVEL		1

/* For client handling */
#define TSD_C_STAGE_NEW		1
#define TSD_C_STAGE_NORMAL	2
#define TSD_C_STAGE_START	3
#define TSD_C_STAGE_PLAY	4
#define TSD_C_STAGE_STOP	5
#define TSD_C_STAGE_CLOSE	6

/* For polling the client */
#define TSD_CLIENT_TIMEOUT	10000

/* gettime in milliseconds */
#define gettime_ms(t) do { \
    struct timeval __tv;        \
    gettimeofday(&__tv, NULL);  \
    t = (__tv.tv_sec*1000) + (__tv.tv_usec/1000); \
} while(0)

/* gettime in microseconds */
#define gettime_us(t) do { \
    struct timeval __tv;        \
    gettimeofday(&__tv, NULL);  \
    t = (__tv.tv_sec*1000000) + (__tv.tv_usec); \
} while(0)

/* Data structures */
struct tsd_client
{
	struct tsd_client *next;
	struct tsd_state *state;	/* pointer to state */

	/* i/o */
	int tcp_sock;
	int udp_sock;
	SSL *ssl;

	int udp_port;

	/* for stats */
	int count;
	unsigned char *buf;

	int byteOrder;
	int stage;	/* basically unused */

	unsigned int isSecure:1;
	unsigned int pad:7;
};

struct tsd_state
{
	int mmapfd;
	int sock;
	fd_set rfds;
	unsigned int bufsize;
	/* list of clients */
	struct tsd_client *clients; /* :TODO: use thinc_list code */
	/* params of the current stream */
	thinc_sndOpen openParams;
	/* for SSL */
	SSL_CTX *ctx;
	/* read and write funcs */
	int (*read)(struct tsd_client *client, void *data, 
			int offset, int count);
	int (*write)(struct tsd_client *client, const void *data, 
			int offset, int count);
	/* mmap regions */
	char *play;
	char *capture;
	char *capture_head;
	char *capture_tail;
	/* our buffer of commands to send to the client */
	unsigned char cmdbuf[256];

	int port;

	thinc_sndUDPData *udp_data;

	/* some flags */
	unsigned int beSecure:1;
	unsigned int playIsOpen:1;
	unsigned int captureIsOpen:1;
	unsigned int udp:1;
	unsigned int pad:4;
};


/* Prototypes */
void tsd_cleanup(struct tsd_state *state);
void tsd_closeClient(struct tsd_client *client);
void tsd_initDriverCmdFuncs(void);
int tsd_initClientUDP(struct tsd_client *client, struct tsd_state *state);
void tsd_handleNetworkActivity(struct tsd_state *state);
void tsd_handleDriverActivity(struct tsd_state *state);
int tsd_initListen(struct tsd_state *state, int port);
int tsd_waitForActivity(struct tsd_state *state);
int tsd_writeToClients(struct tsd_state *state, void *data, int count);
inline int tsd_isOpen(struct tsd_state *state, int which);
unsigned int tsd_driverResetPlayCmds(int fd);

extern char *trace;
extern char *ctrace;

#ifdef TSD_DO_TRACES
#define add_trace() do {						\
	unsigned long __a;						\
	gettime_us(__a);						\
	ctrace += sprintf(ctrace, "%lu\n", __a);			\
} while(0)
#else
#define add_trace()
#endif /* TSD_DO_TRACES */

#endif
